<?php

namespace Drupal\Tests\extra_field_plus_example\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the extra_field_plus_example javascript functionalities.
 *
 * @group extra_field_plus_example
 */
class ExtraFieldPluginPlusExampleFunctionalJavascriptTest extends WebDriverTestBase {

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'node',
    'field',
    'field_ui',
    'test_page_test',
    'extra_field',
    'extra_field_plus',
    'extra_field_plus_example',
  ];

  /**
   * A user with admin permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $adminUser;

  /**
   * A user with authenticated permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $user;

  /**
   * A node.
   *
   * @var \Drupal\node\NodeInterface
   */
  protected $node;


  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->createContentType(['type' => 'article']);
    $this->node = $this->drupalCreateNode([
      'title' => 'VerySpecificTestTitle',
      'type' => 'article',
      'body' => 'Body field value.',
    ]);

    $this->config('system.site')->set('page.front', '/test-page')->save();

    $this->user = $this->drupalCreateUser([]);
    $this->adminUser = $this->drupalCreateUser([]);
    $this->adminUser->addRole($this->createAdminRole('admin', 'admin'));
    $this->adminUser->save();
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests the pseudo field Example Node Label.
   */
  public function testFieldExampleNodeLabel() {
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    $driver = $this->getSession()->getDriver();
    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $display_repository */
    $display_repository = \Drupal::service('entity_display.repository');
    $display_repository->getViewDisplay('node', 'article')
      ->setComponent('extra_field_example_node_label', [
        'type' => 'extra_field_example_node_label',
        'settings' => [
          'link_to_entity' => FALSE,
          'wrapper' => 'span',
        ],
      ])
      ->enable()
      ->save();

    $this->drupalGet('/admin/structure/types/manage/article/display');
    // @todo start from here.
    $page->pressButton('edit-fields-extra-field-example-node-label-settings-edit');
    $session->waitForElementVisible('css', 'input[id*="edit-fields-extra-field-example-node-label-settings-edit-form-actions-save-settings"]');
    $this->submitForm([
      'fields[extra_field_example_node_label][settings_edit_form][settings][link_to_entity]' => FALSE,
      'fields[extra_field_example_node_label][settings_edit_form][settings][wrapper]' => 'span',
    ], 'Update');
    $session->waitForElementRemoved('css', 'input[id*="edit-fields-extra-field-example-node-label-settings-edit-form-actions-save-settings"]');
    $page->pressButton('edit-submit');

    $page = $this->drupalGet('/node/' . $this->node->id());
    $session->elementTextEquals('css', 'span', 'VerySpecificTestTitle');
  }

  /**
   * Tests the pseudo field Example Node Label Formatted.
   */
  public function testFieldExampleNodeLabelFormatted() {
    $session = $this->assertSession();
    $page = $this->getSession()->getPage();
    $driver = $this->getSession()->getDriver();
    /** @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface $display_repository */
    $display_repository = \Drupal::service('entity_display.repository');
    $display_repository->getViewDisplay('node', 'article')
      ->setComponent('extra_field_example_node_label_formatted', [
        'type' => 'extra_field_example_node_label_formatted',
        'settings' => [
          'link_to_entity' => FALSE,
          'wrapper' => 'span',
        ],
      ])
      ->enable()
      ->save();

    $this->drupalGet('/admin/structure/types/manage/article/display');
    // @todo start from here.
    $page->pressButton('edit-fields-extra-field-example-node-label-formatted-settings-edit');
    $session->waitForElementVisible('css', 'input[id*="edit-fields-extra-field-example-node-label-formatted-settings-edit-form-actions-save-settings"]');
    $this->submitForm([
      'fields[extra_field_example_node_label_formatted][settings_edit_form][settings][link_to_entity]' => FALSE,
      'fields[extra_field_example_node_label_formatted][settings_edit_form][settings][wrapper]' => 'span',
    ], 'Update');
    $session->waitForElementRemoved('css', 'input[id*="edit-fields-extra-field-example-node-label-formatted-settings-edit-form-actions-save-settings"]');
    $page->pressButton('edit-submit');

    $page = $this->drupalGet('/node/' . $this->node->id());
    $session->elementTextEquals('css', 'span', 'VerySpecificTestTitle');
  }

}
